#define MOZ_SOURCE_STAMP e81854d6ce91f3174774a50c9c5813c33b9aac58
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-release
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-release/rev/e81854d6ce91f3174774a50c9c5813c33b9aac58
